@@ -147,8 +147,8 @@ def pagination_components_mutator(
                     "name": f"{fld}_after",
                     "in": "query",
                     "required": False,
-                    "schema": {"type": "string", "format": "date-time"},
-                    "description": f"Only items with {fld}_at strictly after this timestamp.",
+                    "schema": {"type": "string"},
+                    "description": f"Only items with {fld}_at strictly after this HTTP-date (RFC 9110).",
                 },
             )
             params.setdefault(
@@ -157,8 +157,8 @@ def pagination_components_mutator(
                     "name": f"{fld}_before",
                     "in": "query",
                     "required": False,
-                    "schema": {"type": "string", "format": "date-time"},
-                    "description": f"Only items with {fld}_at strictly before this timestamp.",
+                    "schema": {"type": "string"},
+                    "description": f"Only items with {fld}_at strictly before this HTTP-date (RFC 9110).",
                 },
             )
 
@@ -1131,20 +1131,183 @@ def drop_unused_components_mutator_auto(keep_schemas: set[str] | None = None):
 
         # schemas
         sch = comps.get("schemas") or {}
-        to_drop = [
-            name
-            for name in sch.keys()
-            if name not in keep_schemas and name not in (used.get("schemas") or set())
-        ]
-        for name in to_drop:
-            sch.pop(name, None)
+        for name in list(sch.keys()):
+            if name not in keep_schemas and name not in (used.get("schemas") or set()):
+                sch.pop(name, None)
 
-        # responses (optional; usually all are used)
+        # responses
         resps = comps.get("responses") or {}
-        to_drop_r = [name for name in resps.keys() if name not in (used.get("responses") or set())]
-        for name in to_drop_r:
-            resps.pop(name, None)
+        for name in list(resps.keys()):
+            if name not in (used.get("responses") or set()):
+                resps.pop(name, None)
 
+        # NEW: headers
+        hdrs = comps.get("headers") or {}
+        for name in list(hdrs.keys()):
+            if name not in (used.get("headers") or set()):
+                hdrs.pop(name, None)
+
+        # (Optional: parameters/requestBodies/etc. same pattern)
+
+        return schema
+
+    return m
+
+
+def hardening_components_mutator():
+    def m(schema: dict) -> dict:
+        schema = dict(schema)
+        comps = schema.setdefault("components", {})
+        params = comps.setdefault("parameters", {})
+        headers = comps.setdefault("headers", {})
+
+        params.setdefault(
+            "IdempotencyKey",
+            {
+                "name": "Idempotency-Key",
+                "in": "header",
+                "required": False,
+                "schema": {"type": "string"},
+                "description": "Provide to make the request idempotent for 24h.",
+            },
+        )
+        params.setdefault(
+            "IfNoneMatch",
+            {
+                "name": "If-None-Match",
+                "in": "header",
+                "required": False,
+                "schema": {"type": "string"},
+                "description": "Conditional GET (ETag).",
+            },
+        )
+        params.setdefault(
+            "IfModifiedSince",
+            {
+                "name": "If-Modified-Since",
+                "in": "header",
+                "required": False,
+                "schema": {"type": "string"},
+                "description": "Conditional GET. HTTP-date per RFC 9110 (e.g. 'Wed, 01 Jan 2025 00:00:00 GMT').",
+            },
+        )
+        params.setdefault(
+            "IfMatch",
+            {
+                "name": "If-Match",
+                "in": "header",
+                "required": False,
+                "schema": {"type": "string"},
+                "description": "Optimistic concurrency for updates.",
+            },
+        )
+
+        headers.setdefault(
+            "ETag", {"schema": {"type": "string"}, "description": "Opaque entity tag."}
+        )
+        headers.setdefault(
+            "LastModified",
+            {
+                "schema": {"type": "string"},  # HTTP-date string
+                "description": "Last modification time, HTTP-date per RFC 9110.",
+            },
+        )
+        headers.setdefault(
+            "XRateLimitLimit", {"schema": {"type": "integer"}, "description": "Tokens in window."}
+        )
+        headers.setdefault(
+            "XRateLimitRemaining",
+            {"schema": {"type": "integer"}, "description": "Remaining tokens."},
+        )
+        headers.setdefault(
+            "XRateLimitReset", {"schema": {"type": "integer"}, "description": "Unix reset time."}
+        )
+        headers.setdefault(
+            "XRequestId", {"schema": {"type": "string"}, "description": "Correlation id."}
+        )
+        headers.setdefault(
+            "Deprecation",
+            {
+                "schema": {"type": "string"},
+                "description": "Set to 'true' for deprecated endpoints.",
+            },
+        )
+        headers.setdefault(
+            "Sunset",
+            {"schema": {"type": "string"}, "description": "HTTP-date for deprecation sunset."},
+        )
+        return schema
+
+    return m
+
+
+def attach_header_params_mutator():
+    def m(schema: dict) -> dict:
+        schema = dict(schema)
+        for _, method, op in _iter_ops(schema):
+            params = op.setdefault("parameters", [])
+
+            def add_ref(name):
+                params.append({"$ref": f"#/components/parameters/{name}"})
+
+            if method in ("post", "patch", "delete"):
+                add_ref("IdempotencyKey")
+                if method in ("patch", "put"):
+                    add_ref("IfMatch")
+            if method == "get":
+                add_ref("IfNoneMatch")
+                add_ref("IfModifiedSince")
+
+            # RESPONSE HEADERS — use *hyphenated* header names as keys
+            resps = op.get("responses") or {}
+            for code, resp in resps.items():
+                try:
+                    ic = int(code)
+                except Exception:
+                    continue
+                if 200 <= ic < 300:
+                    hdrs = resp.setdefault("headers", {})
+                    hdrs.setdefault("ETag", {"$ref": "#/components/headers/ETag"})
+                    hdrs.setdefault("Last-Modified", {"$ref": "#/components/headers/LastModified"})
+                    hdrs.setdefault("X-Request-Id", {"$ref": "#/components/headers/XRequestId"})
+                    hdrs.setdefault(
+                        "X-RateLimit-Limit", {"$ref": "#/components/headers/XRateLimitLimit"}
+                    )
+                    hdrs.setdefault(
+                        "X-RateLimit-Remaining",
+                        {"$ref": "#/components/headers/XRateLimitRemaining"},
+                    )
+                    hdrs.setdefault(
+                        "X-RateLimit-Reset", {"$ref": "#/components/headers/XRateLimitReset"}
+                    )
+                if code == "429":
+                    resp.setdefault("headers", {})["Retry-After"] = {
+                        "schema": {"type": "integer"},
+                        "description": "Seconds until next allowed request.",
+                    }
+        return schema
+
+    return m
+
+
+def attach_conditional_get_304_mutator():
+    def m(schema: dict) -> dict:
+        schema = dict(schema)
+        for _, method, op in _iter_ops(schema):
+            if method != "get":
+                continue
+            resps = op.setdefault("responses", {})
+            resps.setdefault(
+                "304",
+                {
+                    "description": "Not Modified",
+                    "headers": {
+                        "ETag": {"$ref": "#/components/headers/ETag"},
+                        "Last-Modified": {"$ref": "#/components/headers/LastModified"},
+                        "X-Request-Id": {"$ref": "#/components/headers/XRequestId"},
+                    },
+                },
+            )
         return schema
 
     return m
@@ -1158,8 +1321,11 @@ def setup_mutators(
 ) -> list:
     mutators = [
         conventions_mutator(),
+        hardening_components_mutator(),
+        attach_header_params_mutator(),
         normalize_problem_and_examples_mutator(),
         attach_standard_responses_mutator(),
+        attach_conditional_get_304_mutator(),
         auth_mutator(include_api_key),
         strip_ref_siblings_in_responses_mutator(),
         prune_invalid_responses_keys_mutator(),
